#include "humanmanager.h"
#include "accelarrow.h"
#include "cap.h"
#include "circle.h"
#include "game.h"
#include "controlmanager.h"
#include "actormanager.h"
#include "collision.h"
#include "arrow.h"

HumanManager::HumanManager()
{
}

HumanManager::~HumanManager()
{
}

void HumanManager::init(Game::team_t t)
{
	s=(END);
	onemore=false;

	//El HumanManager es del equipo t
	team=t;

	//Creamos un control que controla los clicks
	controller=new Control;
	controller->addActionName(CLICK,"Seleccionar");
	controller->setActionPeripheral(CLICK,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_LBUTTON,Peripheral::ON_PRESS);

	controller->addActionName(UNCLICK,"Despulsar");
	controller->setActionPeripheral(UNCLICK,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_LBUTTON,Peripheral::ON_RELEASE); 

	controller->addActionName(MOVE_X,"Move x");
	controller->setActionPeripheral(MOVE_X,g->\
			controlmanager->mouse,Mouse::MOUSE_X,Peripheral::ON_PRESSING);

	controller->addActionName(MOVE_Y,"Move y");
	controller->setActionPeripheral(MOVE_Y,g->\
			controlmanager->mouse,Mouse::MOUSE_Y,Peripheral::ON_PRESSING);

	controller->addActionName(EXIT,"Salir");
	controller->setActionPeripheral(EXIT,\
			g->controlmanager->kbd,\
			SDLK_ESCAPE,Peripheral::ON_PRESS); 

	//El control controla la flecha
	controller->setOwner(this);
	//Lo anadimos a la lista de controles
	g->controlmanager->add(controller);


	vel_arrow=new Arrow(zero);
	vel_arrow->init();
	accel_arrow=new AccelArrow(zero);
	accel_arrow->init();
	ac=new ActiveCap(zero);
	ac->init();
}

void HumanManager::shutdown()
{
	g->actormanager->erase(vel_arrow);
	g->actormanager->erase(accel_arrow);
	g->actormanager->erase(ac);
	g->controlmanager->del(controller);
}

void HumanManager::stop()
{
	//Si tiene que acabar matamos a la flecha si esta creada
	switch(s)
	{
		case SELECTED:
		case C_ARROW:
			g->actormanager->desactivate(vel_arrow);
			g->actormanager->desactivate(accel_arrow);
			g->actormanager->desactivate(ac);
		case WAIT:
		case END:
			s=END;
	}
}
void HumanManager::doAction(Controllable::action_t act,int magnitude)
{
	Actor *a=NULL;
	//Guardamos donde a clickeado el raton
	Vector mouse;
	mouse.set(g->controlmanager->mouse->get_state(Mouse::MOUSE_X),
			g->controlmanager->mouse->get_state(Mouse::MOUSE_Y));
	
	g->actormanager->rewind();

	switch(s)
	{
		case WAIT:
			if(act==CLICK)
			{
				//Pasamos por todos los actores 
				while((a=g->actormanager->next())!=NULL)
				{
					//Si es una chapa y es de las nuestras
					if(team==a->getTeam() && dynamic_cast<Cap*>(a))
					{
						Cap *c=(Cap*)a;
						//Si ha clickeado dentro de la chapa
						if(isCollision(mouse,c)!=NO_COLLISION)
						{
							//Creamos una flecha unida a la chapa clickeada
							vel_arrow->setPos(c->getPos());
							vel_arrow->setCap(c);
							accel_arrow->setCap(c);
							g->actormanager->activate(vel_arrow);
							//Creamos una aro alrededor de la chapa
							ac->setYellow(c->getYellow());
							ac->setPos(c->getPos());
							g->actormanager->activate(ac);
							//Cogemos el id de la chapa
							capID=c->getID();
							//Pasamos al siguiente estado
							s=SELECTED;
							return;
						}
					}
				}
			}
			break;

		case SELECTED:
			if(act==CLICK)
			{
				while((a=g->actormanager->next())!=NULL && !onemore)
				{
					//Si es una chapa y es de las nuestras
					if(team==a->getTeam() && dynamic_cast<Cap*>(a))
					{
						Cap *c=(Cap*)a;
						//Si ha clickeado dentro de la chapa
						if(isCollision(mouse,c)!=NO_COLLISION)
						{
							//Creamos una flecha unida a la chapa clickeada
							vel_arrow->setPos(c->getPos());
							ac->setYellow(c->getYellow());
							ac->setPos(c->getPos());
							vel_arrow->setCap(c);
							accel_arrow->setCap(c);
							//Cogemos el id de la chapa
							capID=c->getID();
							return;
						}
					}
				}
				//Si no ha clickeado a ninguna chapa empezamos a cargar
				vel_arrow->charge();
				g->actormanager->activate(accel_arrow);

				accel_arrow->charge();
				accel_arrow->setPos(vel_arrow->getPos());
				Vector dir=sub(mouse,vel_arrow->getPos());
				accel_arrow->setDir(dir);
				s=C_ARROW;
			}
			else if(act==MOVE_X || act==MOVE_Y)
			{
				Vector dir=sub(mouse,vel_arrow->getPos());
				vel_arrow->setDir(dir);
			}
			break;

		case C_ARROW:
			if(act==UNCLICK)
			{
				//Guardamos la velocidad, la aceleracion y la direccion
				vel=vel_arrow->getArrow();
				vel=multiply(30,vel);
				accel=accel_arrow->getArrow();
				accel=multiply(3,accel);

				g->actormanager->desactivate(vel_arrow);
				g->actormanager->desactivate(accel_arrow);
				g->actormanager->desactivate(ac);

				onemore=false;
				s=END;
			}
			else if(act==MOVE_X || act==MOVE_Y)
			{
				Vector dir=sub(mouse,accel_arrow->getPos());
				accel_arrow->setDir(dir);
				
			}
			break;
		default:
			break;
	}
	if(act==EXIT)
		g->setEnd();
}

void HumanManager::oneMore(Cap *c)
{
	onemore=true;
	capID=c->getID();

	vel_arrow->setPos(c->getPos());
	vel_arrow->setCap(c);
	accel_arrow->setCap(c);
	g->actormanager->activate(vel_arrow);
	//Creamos una aro alrededor de la chapa
	ac->setYellow(c->getYellow());
	ac->setPos(c->getPos());
	g->actormanager->activate(ac);
	//Cogemos el id de la chapa
	//Pasamos al siguiente estado
	s=SELECTED;
}
