#include "stadium.h"
#include "image.h"
#include "game.h"
#include "soccergame.h"
#include "actormanager.h"
#include "rendermanager.h"
#include "collision.h"
#include "activecap.h"

#include <fstream>
#include <string>

#define SOCCERGAME SG

using namespace std;

Stadium::Stadium(Game *ag):g(ag),goldengoal(false)
{
}

Stadium::~Stadium()
{
}

void Stadium::init()
{
	theme=new Theme("theme.txt");
	cap_pass=NULL;
	cap_fault=NULL;
	finish=false;

	//Los aros alrededor de la chapa
	ActiveCap::setFrames(theme->activecap_frames);

	//Parte logica de la porteria
	int goal_y=theme->stadium_y+(theme->stadium_h-theme->goal_h)/2;

	//Porterias
	g_left=new Goal(SG::LEFT);
	g_left->init();
	//Separamos la parte grafica de la logica
	g_left->setPos(Vector(theme->stadium_x-g_left->get_w()+theme->poster,
				theme->stadium_y+(theme->stadium_h-g_left->get_h())/2));
	g->actormanager->activate(g_left);

	g_right=new Goal(SG::RIGHT);
	g_right->init();
	//Separamos la parte grafica de la logica
	g_right->setPos(Vector(theme->stadium_x+theme->stadium_w-theme->poster,
				theme->stadium_y+(theme->stadium_h-g_right->get_h())/2));
	g->actormanager->activate(g_right);

	//Banda arriba
	Line *l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x,theme->stadium_y));
	l->setSize(Vector(theme->stadium_w,0));
	g->actormanager->activate(l);

	//Banda abajo
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x,theme->stadium_y+theme->stadium_h));
	l->setSize(Vector(theme->stadium_w,0));
	g->actormanager->activate(l);

	//fondo izquierda arriba
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x,theme->stadium_y));
	l->setSize(Vector(0,goal_y-theme->stadium_y));
	g->actormanager->activate(l);

	//fondo izquierda abajo
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x,goal_y+theme->goal_h));
	l->setSize(Vector(0,goal_y-theme->stadium_y));
	g->actormanager->activate(l);

	//porteria izquierda
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x-theme->goal_w,goal_y));
	l->setSize(Vector(0,theme->goal_h));
	g->actormanager->activate(l);

	//porteria izquierda lateral arriba
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x-theme->goal_w,goal_y));
	l->setSize(Vector(theme->goal_w,0));
	g->actormanager->activate(l);

	//porteria izquierda lateral de abajo
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x-theme->goal_w,goal_y+theme->goal_h));
	l->setSize(Vector(theme->goal_w,0));
	g->actormanager->activate(l);

	//fondo derecha arriba
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x+theme->stadium_w,theme->stadium_y));
	l->setSize(Vector(0,goal_y-theme->stadium_y));
	g->actormanager->activate(l);

	//fondo derecha abajo
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x+theme->stadium_w,
				goal_y+theme->goal_h));
	l->setSize(Vector(0,goal_y-theme->stadium_y));
	g->actormanager->activate(l);

	//porteria derecha
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x+theme->stadium_w+theme->goal_w,goal_y));
	l->setSize(Vector(0,theme->goal_h));
	g->actormanager->activate(l);

	//porteria izquierda lateral arriba
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x+theme->stadium_w,goal_y));
	l->setSize(Vector(theme->goal_w,0));
	g->actormanager->activate(l);

	//porteria izquierda lateral de abajo
	l=new Line(SG::NEUTRAL);
	l->init();
	l->setPos(Vector(theme->stadium_x+theme->stadium_w,
				goal_y+theme->goal_h));
	l->setSize(Vector(theme->goal_w,0));
	g->actormanager->activate(l);

	ball=new Ball(zero);
	ball->setInitialPos(Vector(theme->stadium_x+theme->stadium_w/2,
				theme->stadium_y+theme->stadium_h/2));
	ball->returnInitialPos();
	ball->setGraze(1.0-theme->graze);
	ball->setMass(theme->ball_mass);
	ball->init();
	g->actormanager->activate(ball);

	Team::setGame(g);
	teams[SG::LEFT].init(SG::LEFT,theme);
	teams[SG::RIGHT].init(SG::RIGHT,theme);

	//Minutos
	minutes=new TimeBoard(zero);
	minutes->setPos(Vector(theme->time_min_x,theme->time_min_y));
	minutes->countdown(theme->time);
	minutes->setType(TimeBoard::MINUTE);
	minutes->init();
	g->actormanager->activate(minutes);

	//Segundos
	seconds=new TimeBoard(zero);
	seconds->setPos(Vector(theme->time_sec_x,theme->time_sec_y));
	seconds->countdown(theme->time);
	seconds->init();
	g->actormanager->activate(seconds);

	//Tiempo del equipo
	time=new TimeBoard(zero);
	time->setPos(Vector(theme->time_team_x,theme->time_team_y));
	time->countdown(theme->time_team);
	time->init();
	g->actormanager->activate(time);

	//Scores
	s_left=new Score(zero);
	s_left->setPos(Vector(theme->scoreboard_1_goals_x,theme->scoreboard_1_goals_y));
	s_left->init();
	g->actormanager->activate(s_left);

	s_right=new Score(zero);
	s_right->setPos(Vector(theme->scoreboard_2_goals_x,theme->scoreboard_2_goals_y));
	s_right->init();
	g->actormanager->activate(s_right);

	//Select
	s=new Select(zero);
	s->setLeft(Vector(theme->select_left_x,theme->select_left_y));
	s->setRight(Vector(theme->select_right_x,theme->select_right_y));
	s->init();
	g->actormanager->activate(s);

	//Messenger
	messenger=new Messenger(g);
	messenger->init();

	//Pase
	pass=new Pass(zero);
	pass->init();

	changeTeam(SG::LEFT);
	startTurn();
}

void Stadium::shutdown()
{
	teams[SG::LEFT].shutdown();
	teams[SG::RIGHT].shutdown();
	messenger->shutdown();
	g->actormanager->clear();
}

void Stadium::startTurn()
{
	turn=1;
	time->setTime(theme->time_team);
	teams[s->getTeam()].throwCap();
	state=WAITING;
}

bool Stadium::update()
{
	//Si se acaba el tiempo
	if(seconds->isEnd())
	{
		if(!goldengoal)
		{
			if(s_right->getNum()==s_left->getNum())
			{
				goldengoal=true;
				messenger->send(Messenger::GOLDEN);
			}
			else
				finish=true;
		}
	}
	if(teams[SG::LEFT].getNumCaps()==0)
		finish=true;
	if(teams[SG::RIGHT].getNumCaps()==0)
		finish=true;

	switch(state)
	{
		case WAITING:
			if(teams[s->getTeam()].isOver())
				state=MOVING;
			//Si se le ha acabado el tiempo al equipo
			else if(time->isEnd())
			{
				messenger->send(Messenger::TIME);
				teams[s->getTeam()].stop();
				changeTeam();
				startTurn();
			}
			break;

		case MOVING:
			//Si se han dejado de mover y no ha habido gol
			if(!isMoving())
			{
				//Si el equipo no ha gastado todos sus movimientos
				//le damos otro turno
				if(turn<theme->movements)
				{
					turn++;
					teams[s->getTeam()].throwCap();
					state=WAITING;
				}
				else
				{
					changeTeam();
					startTurn();
				}
			}
			break;
		case BALL:
			Cap *aux;
			if(isMoving())
			{
				//Comprobamos que no haya gol del left team
				if(ball->getPos().scale_x()-ball->getRadio()>
						theme->stadium_x+theme->stadium_w)
				{
					changeTeam(SG::RIGHT);
					s_left->goal();
					state=GOAL;
					messenger->send(Messenger::GOAL);
				}
				//Comprobamos que no haya gol del right team
				else if(ball->getPos().scale_x()+ball->getRadio()<theme->stadium_x)
				{
					changeTeam(SG::LEFT);
					s_right->goal();
					state=GOAL;
					messenger->send(Messenger::GOAL);
				}
			}
			//Comprobamos si ha habido pase
			else if((aux=isAPass()) !=NULL)
			{
				messenger->send(Messenger::PASS);
				teams[s->getTeam()].oneMore(aux);
				cap_pass=NULL;
				state=WAITING;
			}
			else
			{
				changeTeam();
				startTurn();
			}
			break;
		case GOAL:
			//Si ha habido gol esperamos a que todo se quede quieto
			if(!isMoving())
			{
				startTurn();
				//Colocamos todo de nuevo empezamos turno
				teams[SG::LEFT].collocateCaps();
				teams[SG::RIGHT].collocateCaps();
				ball->returnInitialPos();
				if(goldengoal==true)
					finish=true;
			}
			break;
		case FAULT:
			//Esperamos a que todo este quieto
			if(!isMoving())
			{
				//Si tiene amarilla
				if(cap_fault->getYellow())
				{
					teams[cap_fault->getTeam()].delCap(cap_fault);
					//Crear animacion tarjeta roja
					messenger->send(Messenger::RED);
				}
				else
				{
					cap_fault->setYellow();
					//Crear animacion tarjeta amarilla
					messenger->send(Messenger::YELLOW);
				}

				cap_fault=NULL;

				startTurn();
				teams[SG::LEFT].collocateCaps();
				teams[SG::RIGHT].collocateCaps();
				ball->returnInitialPos();
			}
			break;
	}
	//Reseteamos la variable de las chapas
	Cap::setAnyMoving();
	return finish;
}

void Stadium::getCollision(Collision &c)
{
	//Comprobamos que el que se esta moviendo sea del equipo del turno
	if(c.a->getTeam()!=s->getTeam())
		return;

	if(c.b->getTeam()!=SG::NEUTRAL)
	{
		if(c.a->getTeam()!=c.b->getTeam())
		{
			if(state==MOVING)
			{
				cap_fault=(Cap*)c.a;
				changeTeam();
				state=FAULT;
			}
		}
	}
	//Si es neutral contra lo que ha chocado
	else
	{
		//Si es redondo es la pelota
		if(c.b->getType()==Actor::CIRCLE)
		{
			if(state==MOVING)
			{
				cap_pass=(Cap*)c.a;
				state=BALL;
				turn=theme->movements;
			}
		}
	}
}

Cap* Stadium::isAPass()
{
	Cap *near=NULL;
	float distance=1024.0;
	for(int i=0;i<theme->caps;i++)
	{
		Cap *c;
		if((c=teams[cap_pass->getTeam()].getCap(i))!=NULL)
		{
			Vector d=Vector(c->getPos()-ball->getPos());
			if(d.module()<distance && cap_pass!=c)
			{
				near=c;
				distance=d.module();
			}
		}
	}
	if(near!=NULL)
	{
		pass->setPos(near->getPos());
		g->actormanager->activate(pass);
		if(isCollision(pass,ball)!=NO_COLLISION)
			return near;
	}
	return NULL;
}
