#ifndef ACTOR_H
#define ACTOR_H

#include <SDL.h>

#include "actorgraphic.h"
#include "game.h"
#include "vector.h"

class Actor
{
	public:
		Actor(){}
		Actor(zero_t):ag(NULL),mass(0),vel(zero),accel(zero),type(BOX),
			collisionable(false),layer(0),team(0){}
		virtual ~Actor(){delete ag;}

		//Iniciar
		virtual void init(){ag->init();}
		//Meter en el juego
		virtual void activate(){ag->activate();}
		//Actualizar
		virtual void update(){move();ag->update();}
		//Sacar del juego
		virtual void desactivate(){ag->desactivate();}

		virtual void move(){}

		enum Type
		{
			BOX,
			CIRCLE,
		};

		//GETS

		Vector getVel()const{return vel;}
		Vector getAccel()const{return accel;}
		float getMass()const{return mass;}

		Type getType()const{return type;}
		int getLayer()const{return layer;}
		Game::team_t getTeam()const{return team;}

		bool isCollisionable()const{return collisionable;}

		virtual Vector getPos()const=0;
		virtual Vector getCenter()const=0;
		virtual SDL_Rect getRect()const=0;
		SDL_Rect getGraphicRect()const{return ag->getRect();}

		virtual int get_h(){return ag->get_h();}
		virtual int get_w(){return ag->get_w();}

		virtual void getCollision(Collision &c){}

		//SETS

		void static setGame(Game *game){g=game;}
		void setActorGraphic(ActorGraphic *g){ag=g;}
		
		void setVel(const Vector &v){vel=v;}
		void setVel(zero_t){vel=Vector(zero);}
		void setAccel(const Vector &a){accel=a;}
		void setAccel(zero_t){accel=Vector(zero);}
		void setVel(const float x,const float y){vel=Vector(x,y);}
		void setMass(const float m){mass=m;}

		void setLayer(const int l){layer=l;}
		void setTeam(const Game::team_t t){team=t;}

		void setCollisionable(const bool d){collisionable=d;}

		virtual void setPos(const Vector &v)=0;


		//Distintas maneras de ordenar los actores
		//Por capas
		static bool actorLayerSort(const Actor* left,const Actor* right)
		{return left->getLayer()<right->getLayer();}


		void draw()const{ag->draw();}
		virtual void draw(SDL_Rect &r)const{ag->draw(r);}

	protected:
		ActorGraphic *ag;
		float mass;
		Vector vel;
		Vector accel;
		Type type;

		bool collisionable;
		int layer;
		Game::team_t team;
		static Game *g;
};
#endif
