#include "actormanager.h"
#include "actor.h"

ActorManager::ActorManager(Game *ag):g(ag)
{
}

ActorManager::~ActorManager()
{
}

void ActorManager::update()
{
	list<Actor*>::iterator tmp;

	//Creamos todos los actores que tiene que crearse
	activateQueue();
	//Actualizamos todos los actores
	for(tmp=actors.begin();tmp!=actors.end();tmp++)
		(*tmp)->update();
	//Eliminamos los actores que ya no son necesarios
	desactivateQueue();
	eraseQueue();
}

void ActorManager::shutdown()
{
	//Borramos los actores que iba a ser borrados
	eraseQueue();
	//Liberamos todos los actores
	for(actors_iter=actors.begin();actors_iter!=actors.end();actors_iter++)
	{
		(*actors_iter)->desactivate();
		delete (*actors_iter);
	}
	actors.clear();
}

void ActorManager::desactivateAll()
{
	for(actors_iter=actors.begin();actors_iter!=actors.end();actors_iter++)
		(*actors_iter)->desactivate();

	actors.clear();
}

void ActorManager::activateQueue()
{
	//Creamos los actores de la lista de los proximamente creados
	if(to_activate.size()>0)
	{
		list<Actor*>::iterator tmp_iter;
		for(tmp_iter=to_activate.begin();tmp_iter!=to_activate.end();tmp_iter++)
		{
			//Comprobamos que no este ya en la lista
			list<Actor*>::iterator tmp_actors_iter=find(actors.begin(),
					actors.end(), *tmp_iter);
			if (tmp_actors_iter==actors.end())
			{
				//Los inicializamos y los anadimos a la verdadera lista
				(*tmp_iter)->activate();
				actors.push_back(*tmp_iter);
			}
		}
		actors.sort(Actor::actorLayerSort);
		
		rewind();
		to_activate.clear();
	}
}

void ActorManager::desactivateQueue()
{
	//Eliminamos a todos los actores que estan en la lista 
	//de los proximamente liberados
	if(to_desactivate.size()!=0)
	{
		list<Actor*>::iterator tmp_iter;
		list<Actor*>::iterator tmp_actors_iter;
		for(tmp_iter=to_desactivate.begin(); tmp_iter!=to_desactivate.end(); tmp_iter++)
		{
			tmp_actors_iter=find(actors.begin(), actors.end(), *tmp_iter);
			if (tmp_actors_iter!=actors.end())
			{
				actors.erase(tmp_actors_iter);
				(*tmp_iter)->desactivate();
			}
		}
		to_desactivate.clear();
	}
}

void ActorManager::eraseQueue()
{
	//Eliminamos a todos los actores que estan en la lista 
	//de los proximamente liberados
	if(to_erase.size()!=0)
	{
		list<Actor*>::iterator tmp_iter;
		list<Actor*>::iterator tmp_actors_iter;
		for(tmp_iter=to_erase.begin(); tmp_iter!=to_erase.end(); tmp_iter++)
		{
			tmp_actors_iter=find(actors.begin(), actors.end(), *tmp_iter);
			if (tmp_actors_iter!=actors.end())
			{
				actors.erase(tmp_actors_iter);
				(*tmp_iter)->desactivate();
			}
			delete (*tmp_iter);
		}
		to_erase.clear();
	}
}

Actor* ActorManager::next()
{
	Actor* tmp;

	//Si es el ultimo devuelve NULL
	if(actors_iter==actors.end())
		return NULL;

	tmp=(*actors_iter);
	actors_iter++;
	return tmp;
}

Actor* ActorManager::current()
{
	//Devuelve el actor apuntado por el puntero y NO avanza una posicion
	//Si es el ultimo devuelve NULL
	if(actors_iter==actors.end())
		return NULL;
	return *actors_iter;
}

bool ActorManager::setPosition(Actor *a)
{
	list<Actor*>::iterator tmp_actors_iter=find(actors.begin(), actors.end(), a);
	if(tmp_actors_iter!=actors.end())
	{
		actors_iter=tmp_actors_iter;
		actors_iter++;
		return true;
	}
	return false;
}
