#include "collision.h"
#include "circularactor.h"
#include "boxactor.h"

//Para pintar
static_t isCollision(const Actor *a,const SDL_Rect &r)
{
	Box b=Box(r.x,r.y,r.w,r.h);

	//Vemos si la imagen colisiona con el rectangulo
	SDL_Rect actorrect=a->getGraphicRect();
	Box actor=Box(actorrect.x,actorrect.y,actorrect.w,actorrect.h);

	return position(&actor,&b);
}

static_t isCollision(const SDL_Rect &r,const SDL_Rect &p)
{
	Box a=Box(r.x,r.y,r.w,r.h);
	Box b=Box(p.x,p.y,p.w,p.h);

	return position(&a,&b);
}

static_t isCollision(const CircularActor *bigger,const CircularActor *smaller)
{return position((Circle*) bigger,(Circle*) smaller);}
static_t isCollision(const BoxActor *bigger,const BoxActor *smaller)
{return position((Box*)bigger,(Box*)smaller);}
static_t isCollision(const CircularActor *bigger,const BoxActor *smaller)
{return position((Circle*)bigger,(Box*)smaller);}
static_t isCollision(const BoxActor *bigger,const CircularActor *smaller)
{return position((Box*)bigger,(Circle*)smaller);}

static_t position(const Circle *bigger,const Circle *smaller)
{
   Vector dif=sub(bigger->getCenter(),smaller->getCenter());
	if(dif.module()>bigger->getRadio()+smaller->getRadio())
		return NO_COLLISION;
	else if(dif.module()+smaller->getRadio()<=bigger->getRadio())
		return IN;
	return COLLISION;
}

static_t position(const Box *bigger,const Box *smaller)
{
   if(bigger->getMin().scale_x() > smaller->getMax().scale_x() || 
				smaller->getMin().scale_x() > bigger->getMax().scale_x()||
            bigger->getMin().scale_y() > smaller->getMax().scale_y()||
				smaller->getMin().scale_y() > bigger->getMax().scale_y()) 
		return NO_COLLISION;

	else if(smaller->getMin().scale_x()>=bigger->getMin().scale_x() &&
			smaller->getMax().scale_x()<=bigger->getMax().scale_x() &&
			smaller->getMin().scale_y()>=bigger->getMin().scale_y() &&
			smaller->getMax().scale_y()<=bigger->getMax().scale_y()) 
		return IN;
	return COLLISION;
}

static_t position(const SDL_Rect &bigger,const SDL_Rect &smaller)
{
	Box b;
	b.locate(Vector(bigger.x,bigger.y));
	b.setSize(Vector(bigger.w,bigger.h));
	Box s;
	s.locate(Vector(smaller.x,smaller.y));
	s.setSize(Vector(smaller.w,smaller.h));
	return position(&b,&s);
}

static_t position(const Circle *bigger,const Box *smaller)
{
/*	//Si estan dentro la esquina superior izq e inferior der
	if(sub(bigger->getCenter(),smaller->getMin()).module() < 
			bigger->getRadio() &&
			sub(bigger->getCenter(),smaller->getMin()+smaller->getSize()) <
			bigger->getRadio()) 
	{
		//Comprobamos las otras dos esquinas
		Vector v1.set(smaller->getMin().scale_x()+smaller->getSize().scale_x(),
				smaller->getMin().scale_y());
		Vector v2.set(smaller->getMin().scale_x()),
				smaller->getMin().scale_y()+smaller->getSize().scale_y();
		if(sub(bigger->getCenter(),v1).module()<bigger->getRadio() &&
				sub(bigger->getCenter(),v2).module()<bigger->getRadio())
			return IN;
	}
	if(bigger->getCenter().scale_x()+bigger->getRadio() <
			smaller->getMin().scale_x() ||
			bigger->getCenter().scale_x()-bigger->getRadio() >
			smaller->getMin().scale_x()+smaller->getSize().scale_x() ||
			bigger->getCenter().scale_y()+bigger->getRadio() <
			smaller->getMin().scale_y() ||
			bigger->getCenter().scale_y()-bigger->getRadio() >
			smaller->getMin().scale_y()+smaller->getSize().scale_y() )
		return NO_COLLISION;
*/	return COLLISION;
}

static_t position(const Box *bigger,const Circle *smaller)
{
	if(smaller->getCenter().scale_x()-smaller->getRadio() >=
			bigger->getMin().scale_x() &&
			smaller->getCenter().scale_x()+smaller->getRadio() <=
			bigger->getMax().scale_x() &&
			smaller->getCenter().scale_y()-smaller->getRadio() >=
			bigger->getMin().scale_y() &&
			smaller->getCenter().scale_y()+smaller->getRadio() <=
			bigger->getMax().scale_y() )
		return IN;
	if(smaller->getCenter().scale_x()+smaller->getRadio() <
			bigger->getMin().scale_x() ||
			smaller->getCenter().scale_x()-smaller->getRadio() >
			bigger->getMax().scale_x() ||
			smaller->getCenter().scale_y()+smaller->getRadio() <
			bigger->getMin().scale_y() ||
			smaller->getCenter().scale_y()-smaller->getRadio() >
			bigger->getMax().scale_y())
		return NO_COLLISION;
	return COLLISION;
}

static_t isCollision(const Vector &v,const Circle *c)
{
   Vector dif=sub(c->getCenter(),v);
   if(dif.module() < c->getRadio())
		return COLLISION;
	return NO_COLLISION;
}

static_t isCollision(const Vector &v,const Box*b)
{
	if(b->getMin().scale_x()>v.scale_x() || b->getMin().scale_y()>v.scale_y() ||
			b->getMax().scale_x()<v.scale_x() || b->getMax().scale_y()<v.scale_y())
		return NO_COLLISION;
	return COLLISION;
}

static_t isCollision(const Vector &v,const CircularActor *c)
{
	return isCollision(v,((Circle*)c));
}

static_t isCollision(const Vector &v,const BoxActor *b)
{
	return isCollision(v,((Box*)b));
}

static_t isCollision(const Vector &p,const Vector &r,const CircularActor *c)
{
   Vector v=add(p,r);
   Vector normal=Vector(r.scale_y(),-r.scale_x());
   normal=normalize(normal);
   normal=multiply(c->getRadio(),normal);
// return isCollision(p,r,c->getCenter(),normal);
   return NO_COLLISION;
}

static_t isCollision(const Vector &v,const Actor *a)
{
	if(a->getType()==Actor::BOX)
	{
		Box *b=((Box*)a);
		return isCollision(v,b);
	}
	Circle *c=((Circle*)a);
	return isCollision(v,c);
}

static_t position(const Vector &p1,const Vector &r1,const Vector &p2,const Vector &r2)
{
   /*
   //Dos rectas definidos como un punto y vector director con modulo
   //de tamanyo de la recta. Sistema de ecuaciones que se debe 
   //resolver: l, k
   // x1 + r1_x * l = x2 + r2_x * k
   // y1 + r1_y * l = y2 + r2_y * k
   
   //Primero comprobamos que no sean paralelos
   if(r1.normalize==r2.normalize)
      return 0;

   //Comprobamos que no sean el mismo punto
   if(p1==p2)
      return 0;

   float l=0;
   float k=0;
   //Vamos a comprobar primero los valores del vector direccion son distinto
   //de 0, porque si lo son no se podra dividir
   if(r1->scale_x()!=0.0)
   {
      l=(p2.scale_x()-p1.scale_x())/r1.scale_x();//mas (r2_x/r1_x)*k
      float aux=r2.scale_y()-r1.scale_y()*r2.scale_x()/r1.scale_x();
      if(aux==0)
      {
         puts("Despejando ecuaciones division entre 0");
         exit(1);
      }
      k=(p1.scale_x()-p2.scale_x()+r1.scale_y()*l)/aux;
      l=l+(r2.scale_x()/r1.scale_x())*k
   }
   else
   {
      k=p1.scale_x()-p2.scale_x()/r2.scale_x();
      if(r1.scale_y()==0.0) //Este caso nunca deberia darse
      {
         puts("Error: vector direccion nulo!!!!!!!!!");
         return 0.0;
      }
      l=(p2.scale_y()-p1.scale_y()+r2.scale_y()*k)/r1.scale_y();
   }
   */
   return NO_COLLISION;
}

SDL_Rect getRectCollision(const SDL_Rect &a,const SDL_Rect &b)
{
	Box aux=Box(a.x,a.y,a.w,a.h);
	Box aux2=Box(b.x,b.y,b.w,b.h);
	return getRectCollision(&aux,&aux2);
}

SDL_Rect getRectCollision(const Box *a,const Box *b)
{
	SDL_Rect aux;

	//Cogemos el x mas grande
	if(a->getMin().scale_x()>=b->getMin().scale_x())
		aux.x=a->getMin().scale_x();
	else
		aux.x=b->getMin().scale_x();

	//Cogemos la posicion x menos alejada y le restamos
	//nuestra x comprobando que no sea negativa
	if(a->getMax().scale_x()<=b->getMax().scale_x())
	{
		aux.w=a->getMax().scale_x()-aux.x;
	}
	else
	{
		aux.w=b->getMax().scale_x()-aux.x;
	}

	//Lo mismo para la y
	if(a->getMin().scale_y()>=b->getMin().scale_y())
		aux.y=a->getMin().scale_y();
	else
		aux.y=b->getMin().scale_y();

	if(a->getMax().scale_y()<=b->getMax().scale_y())
	{
		aux.h=a->getMax().scale_y()-aux.y;
	}
	else
	{
		aux.h=b->getMax().scale_y()-aux.y;
}

	return aux;
}
