#include "collisionmanager.h"
#include "actormanager.h"
#include "circularactor.h"
#include "boxactor.h"

void CollisionManager::init(){}
void CollisionManager::update(){timeOfCollision(1.0);}
void CollisionManager::shutdown(){}

//Calcula el tiempo minimo de colision
void CollisionManager::timeOfCollision(const float t_max=1.0)
{
	//Variables donde guardamos la colision que se realiza antes
	Collision before;
	Collision aux;
	aux.a=before.a=NULL;
	aux.b=before.b=NULL;
	aux.t=before.t=t_max;

	g->actormanager->rewind();
	//Vamos a calcular el tiempo de la primera colision
	while((aux.a=g->actormanager->next())!=NULL)
	{
		//Si el actor no se mueve nos lo saltamos
		if(aux.a->getVel().module()==0.0 || !aux.a->isCollisionable())
			continue;

		//comparamos todos los movimientos con todos los demas actores
		g->actormanager->rewind();
		while((aux.b=g->actormanager->next())!=NULL)
		{
			//Si no es collisionable o es el mismo actor nos lo saltamos
			if(!aux.b->isCollisionable() || aux.b==aux.a)
				continue;

			//Calculamos cuanto tardan en colisionar y tipo de colision
			aux=collision(aux);
			//Si es menor que la mejor solucion actual
			if(aux.t<before.t)
				before=aux;
		}
		//Ponemos el puntero de los actores donde estaba
		g->actormanager->setPosition(aux.a);
	}
	
	//Si no hay ninguna collision t=t_max
	if(before.t==t_max)
		//movemos todo sin problemas
		move(t_max);
	else
	{
		move(before);
		//Vemos cuando se produce la proxima colision
		timeOfCollision(t_max-before.t);
	}
}

//Mueve los actores hasta un tiempo t
void CollisionManager::move(const float t)
{
	Actor* a;
	g->actormanager->rewind();
	while((a=g->actormanager->next())!=NULL)
	{
		a->setPos(add(a->getPos(),multiply(t,a->getVel())));
	}
}

//Modifica 
void CollisionManager::move(Collision &c)
{
	//Movemos todas las chapas justo hasta el momento del
	//choque
	move(c.t);

	//Le pasamos al game la collision
	g->getCollision(c);

	//Primero la golpeada
	c.b->getCollision(c);
	c.a->getCollision(c);
/*	
	CircularActor *cir=(CircularActor*)c.a;
	//Dos tipos de colisiones circulo con circulo
	if(c.b->getType()==Actor::CIRCLE)
	{
		CircularActor *other=(CircularActor*)c.b;

		if(c.type!=ROUND)
			return;
		//Ahora modificamos las velocidades de los actores que
		//colisionan
		//
		//Direccion de la bola que esta parada es el vector
		//diferencia entre los vectores posicion parado y
		//moviendose
		Vector par=sub(other->getPos(),cir->getPos());

		//Direccion de la bola que se esta moviendo es el
		//inverso del vector reflejo respecto a la velparado
		
		Vector mov=reflect(cir->getVel(),par);
		mov=reverse(mov);

		//Ahora calculamos su tamanyo
		//Primero normalizamos los vectores
		mov=normalize(mov);
		par=normalize(par);
		
		//Ahora lo multiplicamos por la cantidad de moviento
		//y una constante que en cada es cos alpha o sen alpha
		float cos_alpha=escalate(mov,par);
		//Nos interesa en valor absoluto
		if(cos_alpha<0)
			cos_alpha=(-cos_alpha);
		float size_mov=(1-cos_alpha)*cir->getVel().module();
		float size_par=cos_alpha*cir->getVel().module()*cir->getMass()/
			other->getMass();

		mov=multiply(size_mov,mov);
		par=multiply(size_par,par);

		//Modificamos sus velocidades
		cir->setVel(mov);
		other->setVel(par);
	}
	//Choque circulo con linea
	else
	{
		BoxActor *line=(BoxActor*)c.b;
		if(c.type==WALL)
		{
			//Vector normal
			Vector n=line->getSize();
			n=normalize(n);
			n=rotateRight(n);
			
			//El nuevo vector de velocidad del circulo es el inverso del reflejo 
			//del actual respecto a la normal de linea
			Vector vel=reflect(cir->getVel(),n);
			vel=reverse(vel);
			cir->setVel(vel);
		}
		else if(c.type==CORNER)
		{
			//Vemos cual es el que esta mas cerca
			Vector dif1=cir->getPos()-line->getMin();
			Vector dif2=cir->getPos()-line->getMax();
			if(dif1.module()<dif2.module())
			{
				dif1=normalize(dif1);
				dif1=multiply(cir->getVel().module(),dif1);
				cir->setVel(dif1);
			}
			else
			{
				dif2=normalize(dif2);
				dif2=multiply(cir->getVel().module(),dif2);
				cir->setVel(dif2);
			}
		}
	}
*/
}

Collision CollisionManager::collision(Collision &c)
{
	CircularActor *a=(CircularActor*)c.a;
	if(c.b->getType()==Actor::CIRCLE)
	{
		CircularActor *b=(CircularActor*)c.b;
		return collision(a,b);
	}
	else
	{
		BoxActor *b=(BoxActor*)c.b;
		return collision(a,b);
	}
}

//Devuelve el tiempo de collision
Collision CollisionManager::collision(const CircularActor* move_cir, const CircularActor *stop_cir)
{
	Collision c;
	c.a=(Actor*)move_cir;
	c.b=(Actor*)stop_cir;
	c.type=ROUND;
	c.t=NO_COLLISION;
	//Vamos a susponer que stop_cir esta parado y move_cir
	//se mueve a la velocidad de ambos
	Vector pos=sub(move_cir->getPos(),stop_cir->getPos());
	Vector vel=sub(move_cir->getVel(),stop_cir->getVel());
	float r=stop_cir->getRadio()+move_cir->getRadio();

	//Ecuacion de segundo grado cuya a,b y d son:
	float a=vel.scale_x()*vel.scale_x()+vel.scale_y()*vel.scale_y();
	float b=2*(pos.scale_x()*vel.scale_x()+pos.scale_y()*vel.scale_y());
	float d=pos.scale_x()*pos.scale_x()+pos.scale_y()*pos.scale_y()-r*r;
	

	//Resolvemos la ecuacion y nos dara un t que es el tiempo
	float aux=(b*b)-(4*a*d);
//Si existen soluciones reales
	if(aux>=0)
	{
		aux=sqrt(aux);
		float sol=(-b-aux)/(2*a);
		//Si esta en el intervalo
		if(sol>0.0001 && sol<=1.0)
			c.t=sol;
		else
		{
			sol=(-b+aux)/(2*a);
			if(sol>0.0001 && sol<=1.0)
				c.t=sol;
		}
	}
	return c;
}

Collision CollisionManager::collision(const CircularActor* move_cir,const BoxActor *line)
{
	Collision c;
	c.a=(Actor*)move_cir;
	c.b=(Actor*)line;
	c.t=NO_COLLISION;
	c.type=WALL;
	//Obtenemos el vector director de la recta
	Vector dir=line->getSize();
	dir=normalize(dir);
	//El vector normal a la recta
	Vector n=rotateRight(dir);

	//La interseccion de la recta que tenemos con una recta
	//que pasa por 0,0 y normal a este. Hallamos c para las
	//ecuaciones de despues
	//n_x*c=p_x+v_x*k
	//n_y*c=p_y+v_y*k
	float d;
	//Despejando
	if(dir.scale_x()!=0)
	{
		d=line->getPos().scale_y()/n.scale_y()-(dir.scale_y()*
				line->getPos().scale_x())/(dir.scale_x()*n.scale_y());
		d=d/(1-(dir.scale_y()*n.scale_x())/
				(dir.scale_x()*n.scale_y()));
	}
	else
		d=line->getPos().scale_x()/n.scale_x();
	
	//Consideramos d*n es el origen de cordenadas creando
	//un sistema con misma solucion que la real pero mas 
	//sencillo de calcular
	//Vector posicion relativa del circulo en el nuevo
	//sistema de coordenadas
	Vector p=sub(move_cir->getPos(),multiply(d,n));
	//Tenemos cuatro soluciones distintas:
	//que choque en el interior sumandole el radio
	//que choque en el interior restandole el radio
	//que choque con un extremo
	//o que choque con el otro
	float sol[4];
	sol[0]=(move_cir->getRadio()-escalate(n,p))/escalate(n,move_cir->getVel());
	sol[1]=(-move_cir->getRadio()-escalate(n,p))/escalate(n,move_cir->getVel());
	sol[2]=CollisionManager::collision(move_cir,line->getPos()).t;
	sol[3]=CollisionManager::collision(move_cir,line->getMax()).t;

	for(int i=0;i<4;i++)
	{
		if(sol[i]<0.0001 || sol[i]>1.0)
			sol[i]=2.0;
	}

	if(sol[0]!=2.0)
	{
		Vector collision=Vector(zero);
		//El radio hay que restarlo
		collision=add(move_cir->getPos(),multiply(sol[0],move_cir->getVel()));
		collision=collision-multiply(move_cir->getRadio(),n);

		//Para comprobar si el punto esta en el segmento obtenemos
		//el vector diferencia entre vector collision 
		//y el punto medio del segmento;
		//si resulta ser mayor en modulo que la mitad del tamanyo
		//del segmento significa que no chocara
		Vector med=line->getPos()+multiply(0.5,line->getSize());
		//med+dif=collision
		Vector dif=collision-med;

		//Si no choca
		if(dif.module()>line->getSize().module()/2)
			sol[0]=2.0;
	}
	//Lo mismo para la otra solucion pero cambiando el vector collision
	if(sol[1]!=2.0)
	{
		Vector collision=Vector(zero);
		//El radio hay que sumarlo
		collision=add(move_cir->getPos(),multiply(sol[1],move_cir->getVel()));
		collision=collision+multiply(move_cir->getRadio(),n);

		Vector med=line->getPos()+multiply(0.5,line->getSize());
		Vector dif=collision-med;
		//Si no choca
		if(dif.module()>line->getSize().module()/2)
			sol[1]=2.0;
	}

	for(int i=0;i<4;i++)
	{
		if(sol[i]>0.0001 && sol[i]<1.0 &&sol[i]<c.t)
		{
			c.t=sol[i];
			if(i<2)
				c.type=WALL;
			else
				c.type=CORNER;
		}
	}
	return c;
}

Collision CollisionManager::collision(const CircularActor* cir,const Vector &point)
{
	CircularActor p=CircularActor(zero);
	p.setPos(point);
	return collision(cir,&p);
}
