#include <SDL.h>
#include "peripheral.h"
#include "control.h"
#include "controllable.h"

void Control::addActionName(Controllable::action_t act,string str)
{
	association_t asoc;

	asoc.act=act;
	asoc.name=str;
	asoc.peri=NULL;
	asoc.comp=-1;
	associations.push_back(asoc);
}

void Control::setActionPeripheral(Controllable::action_t act,
		Peripheral* peri, Peripheral::component_t comp,
		Peripheral::event_t e)
{
	for(associations_iter=associations.begin();
			associations_iter!=associations.end();
			associations_iter++)
	{
		if(associations_iter->act==act)
		{
			associations_iter->peri=peri;
			associations_iter->comp=comp;
			associations_iter->event=e;
			return;
		}
	}
}

void Control::update()
{
	bool do_action_order;
	Peripheral::event_t tmp_event;
	for(associations_iter=associations.begin();
			associations_iter!=associations.end();
			associations_iter++)
	{
		tmp_event=associations_iter->peri->get_event(associations_iter->comp);
		do_action_order=false;

		switch(associations_iter->event)
		{
			case Peripheral::ON_PRESSING:
				if(tmp_event==Peripheral::ON_PRESS)
					do_action_order=true;
				break;

			case Peripheral::ON_PRESS:
				if((tmp_event==Peripheral::ON_PRESS) && \
						associations_iter->peri->is_change(associations_iter->comp))
					do_action_order=true;
				break;

			case Peripheral::ON_RELEASSING:
				if(tmp_event==Peripheral::ON_RELEASE)
				{
					do_action_order=true;
				}
				break;

			case Peripheral::ON_RELEASE:
				if((tmp_event==Peripheral::ON_RELEASE) &&\
						associations_iter->peri->is_change(associations_iter->comp))
					do_action_order=true;
				break;

			default:
				break;
		}
		if(do_action_order)
		{
			owner->doAction(associations_iter->act,\
					associations_iter->peri->get_state(associations_iter->comp));
		}
	}
}

string Control::getNameAction(Controllable::action_t act)
{
	for(associations_iter=associations.begin();
			associations_iter!=associations.end();
			associations_iter++)
	{
		if(associations_iter->act==act)
		{
			return associations_iter->name;
		}
	}
	return "";
}

