#include <SDL.h>
#include <vector>

#include "peripheral.h"
#include "controlmanager.h"
#include "mouse.h"
#include "keyboard.h"

ControlManager::ControlManager(Game *g): game(g)
{
}

ControlManager::~ControlManager()
{
}

void ControlManager::init()
{
	kbd=new Keyboard();
	kbd->init();
	mouse=new Mouse();
	mouse->init();
}

void ControlManager::update()
{
	//Primero actualizamos los old_event de los perifericos
	kbd->update();
	mouse->update();

	//Actualizamos los perifericos
	get_events();

	//Vemos si afecta a algun control
	for(controls_iter=controls.begin();
			controls_iter!=controls.end();controls_iter++)
		(*controls_iter)->update();
}

void ControlManager::shutdown()
{
	for(controls_iter=controls.begin();
			controls_iter!=controls.end();controls_iter++)
	{
		delete(*controls_iter);
	}
	delete kbd;
	delete mouse;
}

int ControlManager::add(Control *c)
{
	controls.push_back(c);
	return controls.size();
}

int ControlManager::del(Control *c)
{
	list<Control*>::iterator tmp_iter;

	tmp_iter=find(controls.begin(),controls.end(),c);
	if(tmp_iter!=controls.end())
	{
		controls.erase(tmp_iter);
		delete (*tmp_iter);
	}

	return controls.size();
}

void ControlManager::get_events()
{
	SDL_Event event;
	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			//Quit event
			case SDL_QUIT:
				game->setEnd();
				break;

			//Keyboard's events
			case SDL_KEYDOWN:
				kbd->set_component(event.key.keysym.sym,true);
				break;
			case SDL_KEYUP:
				kbd->set_component(event.key.keysym.sym,false);
				break;

			//Mouse's events
			case SDL_MOUSEBUTTONDOWN:
				mouse->set_component(event.button.button,true);
				break;
			case SDL_MOUSEBUTTONUP:
				mouse->set_component(event.button.button,false);
				break;

			case SDL_MOUSEMOTION:
				mouse->set_component(Mouse::MOUSE_X,event.motion.x);
				mouse->set_component(Mouse::MOUSE_Y,event.motion.y);
				break;
				
			default:
				break;
		}
	}
}			
