#include <SDL.h>
#include <string>
#include <iostream>

using namespace std;

#include "actormanager.h"
#include "rendermanager.h"
#include "timemanager.h"
#include "controlmanager.h"
#include "collisionmanager.h"
#include "fontmanager.h"
#include "game.h"

Game::Game(): actormanager(NULL),rendermanager(NULL),timemanager(NULL),\
					 controlmanager(NULL),collisionmanager(NULL),
					 fontmanager(NULL),end(false),name(""),icon("")
{
}

Game::~Game()
{
}

int Game::init(const int w,const int h,const int col)
{
   if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER)<0)
   {
      cerr<<"No pudo inicializarse SDL: "<<SDL_GetError()<<endl;
      exit(-1);
   }
	create_actormanager();
	create_rendermanager(w,h,col);
	create_timemanager();
	create_controlmanager();
	create_collisionmanager();
	create_fontmanager();
	SDL_WM_SetCaption(name.c_str(),icon.c_str());
	start();

	return returnvalue;
}

void Game::update()
{
	if(timemanager->isLogictick())
	{
		logicgame();
		controlmanager->update();
		timemanager->update();
		collisionmanager->update();
		fontmanager->update();
		actormanager->update();
	}
	if(timemanager->isGraphictick())
		rendermanager->update();
}

void Game::shutdown(string message)
{
	unload();
	if(actormanager)
	{
		actormanager->shutdown();
		delete actormanager;
	}
	if(rendermanager)
	{
		rendermanager->shutdown();
		delete rendermanager;
	}
	if(timemanager)
	{
		timemanager->shutdown();
		delete timemanager;
	}
	if(controlmanager)
	{
		controlmanager->shutdown();
		delete controlmanager;
	}
	if(collisionmanager)
	{
		collisionmanager->shutdown();
		delete collisionmanager;
	}
	if(fontmanager)
	{
		fontmanager->shutdown();
		delete fontmanager;
	}

	SDL_Quit();

	cout<<name<<endl;
	cout<<message<<endl;
}

void Game::create_actormanager()
{
	actormanager=new ActorManager(this);
	actormanager->init();
}

void Game::create_rendermanager(const int w,const int h,const int col)
{
	rendermanager=new RenderManager(this);
	rendermanager->init(w,h,col);
}

void Game::create_timemanager()
{
	timemanager=new TimeManager(this);
	timemanager->init(20);
}

void Game::create_controlmanager()
{
	controlmanager=new ControlManager(this);
	controlmanager->init();
}

void Game::create_collisionmanager()
{
	collisionmanager=new CollisionManager(this);
	collisionmanager->init();
}

void Game::create_fontmanager()
{
	fontmanager=new FontManager(this);
	fontmanager->init();
}

void Game::start()
{
	load();
	main();
	shutdown();
}

void Game::main()
{
	while(!end)
		update();
}

void Game::setName(const string n)
{
	name=n;
	SDL_WM_SetCaption(name.c_str(),icon.c_str());
}

void Game::setIcon(const string n)
{
	icon=n;
	SDL_WM_SetCaption(name.c_str(),icon.c_str());
}
