#include <SDL.h>

#include "menucontroller.h"
#include "menuelement.h"
#include "controlmanager.h"
#include "actormanager.h"
#include "collision.h"
#include "menu.h"

void MenuController::init()
{
	controller=new Control();

	controller->addActionName(UP,"Arriba");
	controller->setActionPeripheral(UP, 
			g->controlmanager->kbd,SDLK_UP,Peripheral::ON_PRESS);

	controller->addActionName(DOWN,"Abajo");
	controller->setActionPeripheral(DOWN,
			g->controlmanager->kbd,SDLK_DOWN,Peripheral::ON_PRESS);
	
	controller->addActionName(LEFT,"Izquierda");
	controller->setActionPeripheral(LEFT,
			g->controlmanager->kbd,SDLK_LEFT,Peripheral::ON_PRESS);

	controller->addActionName(RIGHT,"Derecha");
	controller->setActionPeripheral(RIGHT,
			g->controlmanager->kbd,SDLK_RIGHT,Peripheral::ON_PRESS);

	controller->addActionName(ENTER,"Enter");
	controller->setActionPeripheral(ENTER,
			g->controlmanager->kbd,SDLK_RETURN,Peripheral::ON_PRESS);

	controller->addActionName(CLICK,"Click");
	controller->setActionPeripheral(CLICK,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_LBUTTON,Peripheral::ON_PRESS);

	controller->addActionName(UNCLICK,"Unclick");
	controller->setActionPeripheral(UNCLICK,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_LBUTTON,Peripheral::ON_RELEASE);

	controller->addActionName(MOVE_X,"Mouse x");
	controller->setActionPeripheral(MOVE_X,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_X,Peripheral::ON_PRESSING);

	controller->addActionName(MOVE_Y,"Mouse y");
	controller->setActionPeripheral(MOVE_Y,\
			g->controlmanager->mouse,\
			Mouse::MOUSE_Y,Peripheral::ON_PRESSING);

	controller->addActionName(QUIT,"Quit");
	controller->setActionPeripheral(QUIT,
			g->controlmanager->kbd,SDLK_ESCAPE,Peripheral::ON_PRESS); 

	controller->setOwner(this);

	//Lo anadimos a la lista de controles
	g->controlmanager->add(controller);
}

void MenuController::shutdown()
{
	g->controlmanager->del(controller);
}

void MenuController::doAction(Controllable::action_t act,int mag)
{
	Vector mouse;
	mouse.set(g->controlmanager->mouse->get_state(Mouse::MOUSE_X),
		g->controlmanager->mouse->get_state(Mouse::MOUSE_Y));
	MenuMouseElement* m;

	switch(act)
	{
		case MOVE_X:
		case MOVE_Y:
			list->selectedOut();
			list->rewindMouse();
			if(el_onmouse!=NULL)
			{
				if(!el_onmouse->isHere(mouse))
				{
					el_onmouse->getMouseSignal(MenuMouseElement::ON_MOUSEOUT);
					el_onmouse=NULL;
					el_click=NULL;
				}
				else
				{
					return;
				}
			}
			for(m=list->nextMouse();m!=NULL;m=list->nextMouse())
			{
				if(m->isHere(mouse))
				{
					m->getMouseSignal(MenuMouseElement::ON_MOUSEOVER);
					el_onmouse=m;
					break;
				}
			}
			break;

		case CLICK:
			if(el_onmouse!=NULL)
			{
				el_onmouse->getMouseSignal(MenuMouseElement::CLICKED);
				el_click=el_onmouse;
			}
			break;

		case UNCLICK:
			if(el_click!=NULL)
			{
				el_click->getMouseSignal(MenuMouseElement::UNCLICKED);
				el_click=NULL;
			}
			break;

		case ENTER:
			list->pressEnter();
			break;

		case UP:
			list->goUp();
			break;

		case DOWN:
			list->goDown();
			break;

		case LEFT:
			list->goLeft();
			break;

		case RIGHT:
			list->goRight();
			break;

		case QUIT:
			g->setEnd();
			break;
	}
}
