#include <SDL.h>

#include "mouse.h"
#include "peripheral.h"

Mouse::Mouse()
{
}

Mouse::~Mouse()
{
}

void Mouse::init()
{
	reset();
}

void Mouse::update()
{
	old_x=x;
	old_y=y;
	for(int i=0;i<MOUSEBUTTONS;i++)
		old_buttons[i]=buttons[i];
}

void Mouse::shutdown()
{
}


//Devuelve el estado ON_PRESS o ON_RELEASE
Peripheral::event_t Mouse::get_event(Peripheral::component_t comp)
{
	switch(comp)
	{
		case MOUSE_X:
			if(is_change(comp)) 
				return ON_PRESS;
			return ON_RELEASE;
		case MOUSE_Y:
			if(is_change(comp))
				return ON_PRESS;
			return ON_RELEASE;
		default:
			if(buttons[comp])
				return ON_PRESS;
			return ON_RELEASE;
	}
}

//Devuelve el estado de un componente
Peripheral::state_t Mouse::get_state(Peripheral::component_t comp)
{
	switch(comp)
	{
		//Mouse position
		case MOUSE_X:
			return x;
		case MOUSE_Y:
			return y;
		default:
			if(comp<MOUSEBUTTONS)
				return buttons[comp];
			return false;
	}
}


//Devuelve si un ha cambiado respecto a la ultima vez
//Se usa para eventos del tipo ON_PRESS
bool Mouse::is_change(Peripheral::component_t comp)
{
	switch(comp)
	{
		//Mouse position
		case MOUSE_X:
			return x!=old_x;
		case MOUSE_Y:
			return y!=old_y;
		default:
			if(comp<MOUSEBUTTONS)
				return buttons[comp]!=old_buttons[comp];
			return false;
	}
}

void Mouse::set_component(Peripheral::component_t comp,state_t s)
{
	switch(comp)
	{
		//Mouse position
		case MOUSE_X:
			x=s;
			break;
		case MOUSE_Y:
			y=s;
			break;

		//Mouse's buttons
		default:
			if(comp<MOUSEBUTTONS)
				buttons[comp]=s;
			break;
	}
}

void Mouse::reset()
{
	for(int i=0;i<MOUSEBUTTONS;i++)
	{
		buttons[i]=0;
		old_buttons[i]=0;
	}
	SDL_GetMouseState(&x,&y);
	old_x=0;
	old_y=0;
}

string Mouse::get_component_name(Peripheral::component_t comp)
{
	switch(comp)
	{
		//Mouse position
		case Mouse::MOUSE_X:
				return "X del Mouse";
		case Mouse::MOUSE_Y:
				return "Y del Mouse";

		//Mouse's buttons
		case Mouse::MOUSE_LBUTTON:
				return "Boton Izquierdo";
		case Mouse::MOUSE_RBUTTON:
				return "Boton Derecho";
		case Mouse::MOUSE_MBUTTON:
				return "Boton del medio";

		//Wheel
		case Mouse::MOUSE_UP:
				return "Subir la rueda";
		case Mouse::MOUSE_DOWN:
				return "Bajar la rueda";
		default:
			return ""; 
	}
}
