#include <SDL.h>
#include <iostream>

using namespace std;

#include "game.h"
#include "actormanager.h"
#include "rendermanager.h"
#include "utils.h" 
#include "collision.h"
#include <SDL_gfxPrimitives.h>

RenderManager::RenderManager(Game *g): game(g),background(NULL)
{
}

RenderManager::~RenderManager()
{
}

void RenderManager::init(const int w,const int h,const int col)
{
	width=w;
	height=h;
	depth=col;
	n_rects=0;

	//Inicializamos la pantalla
	screen=SDL_SetVideoMode(width,height,depth,SDL_HWSURFACE | SDL_ANYFORMAT |\
			SDL_DOUBLEBUF | SDL_SRCCOLORKEY | SDL_SRCALPHA );

	if(screen==NULL)
	{
		cerr<<"No se puede establecer el modo de pantalla "<<width<<\
			"x"<<height<<":"<<depth<<":"<<SDL_GetError()<<endl;
		exit(1);
	}

	if(SDL_SWSURFACE==(screen->flags & SDL_SWSURFACE))
	{
		dirty_rectangle=true;
		//dirty_rectangle=false;
	}
	if(SDL_HWSURFACE==(screen->flags & SDL_HWSURFACE))
	{
		dirty_rectangle=true;
	}
}

void RenderManager::update()
{
	draw();
}

void RenderManager::shutdown()
{
	SDL_FreeSurface(screen);
	if(background!=NULL)
		SDL_FreeSurface(background);
}

void RenderManager::setBackground(const char* name)
{
	if(background!=NULL)
		SDL_FreeSurface(background);
	background=LoadImg(name);
	drawAll();
}

void RenderManager::addToDraw(const SDL_Rect &r)
{
	SDL_Rect aux;
	SDL_Rect update=r;
	if(update.x<0)
	{
		update.w=update.w-update.x;
		update.x=0;
	}
	if(update.y<0)
	{
		update.h=update.h-update.y;
		update.y=0;
	}
	if(update.x+update.w>SDL_GetVideoSurface()->w)
		update.w=update.w-(update.x+update.w-SDL_GetVideoSurface()->w);
	if(update.y+update.h>SDL_GetVideoSurface()->h)
		update.h=update.h-(update.y+update.h-SDL_GetVideoSurface()->h);

	//printf("n_rects %d\n",n_rects);
	//printf("%d %d %u %u\n",r.x,r.y,r.w,r.h);
	if(n_rects<MAX_RECTS)
	{
		for(unsigned i=0;i<n_rects;i++)
		{
			//Si el rectangulo esta contenido dentro
			//de otro mayor
			if(position(update,to_draw[i])==IN)
			{
				n_rects--;
				for(unsigned j=i;j<n_rects;j++)
				{
					to_draw[j]=to_draw[j+1];
				}
			}
			else if(position(to_draw[i],update)==IN)
				return;
			else if(position(update,to_draw[i])==COLLISION)
			{
				//Entra en collision
				if(update.x<to_draw[i].x)
					aux.x=update.x;
				else
					aux.x=to_draw[i].x;

				if(update.y<to_draw[i].y)
					aux.y=update.y;
				else
					aux.y=to_draw[i].y;

				if(update.x+update.w>to_draw[i].x+to_draw[i].w)
					aux.w=update.x+update.w-aux.x;
				else
					aux.w=to_draw[i].x+to_draw[i].w-aux.x;

				if(update.y+update.h>to_draw[i].y+to_draw[i].h)
					aux.h=update.y+update.h-aux.y;
				else
					aux.h=to_draw[i].y+to_draw[i].h-aux.y;

				//Borramos el rectangulo que acabamos de borrar
				for(unsigned j=i;j<n_rects;j++)
				{
					to_draw[j]=to_draw[j+1];
				}
				n_rects--;
				addToDraw(aux);
				return;
			}
		}
		to_draw[n_rects]=update; 
		n_rects++;
	}
}

void RenderManager::draw()
{
	if(dirty_rectangle && n_rects<MAX_RECTS)
		drawDR();
	else
		drawAll();
}

void RenderManager::drawAll()
{
	//Ponemos el fondo
	SDL_BlitSurface(background,NULL,screen,NULL);

	//Ponemos el resto de actores
	Actor *tmp;
	game->actormanager->rewind();
	while((tmp=game->actormanager->next())!=NULL)
		tmp->draw();

	//Borramos los datos para el dirty_rectangle
	n_rects=0;

	//Actualizamos la pantalla
	SDL_Flip(screen);
}

void RenderManager::drawDR()
{
	Actor *tmp;
	//printf("pintar nrects: %d\n",n_rects);
	for(unsigned i=0;i<n_rects;i++)
	{
	//	printf("%d %d %u %u\n",to_draw[i].x,to_draw[i].y,to_draw[i].w,to_draw[i].h);
		game->actormanager->rewind();
		//Sustituimos la parte modificada por el fondo
		SDL_BlitSurface(background,&to_draw[i],screen,&to_draw[i]);
		//Preguntamos a todos los actores si estan dentro de ese
		//rectangulo
		while((tmp=game->actormanager->next())!=NULL)
		{
			if(isCollision(tmp,to_draw[i])!=NO_COLLISION)
			{
				tmp->draw(to_draw[i]);
			}
		}
	}

	if(n_rects!=0)
	{
		//Actualizamos todas las partes modificadas
		SDL_UpdateRects(SDL_GetVideoSurface(),n_rects,to_draw);
		n_rects=0;
	}
}
