#include <SDL.h>

#include <string>

using namespace std;

#include "sprite.h"
#include "utils.h"
#include "collision.h"
#include "rendermanager.h"


void Sprite::init()
{
	if(img==NULL)
		img=LoadImgAlpha(name);

}

void Sprite::activate()
{
	rect=owner->getRect();
	rect.w=get_w();
	rect.h=get_h();

	g->rendermanager->addToDraw(rect);
}

void Sprite::update()
{
	if(rect.x!=owner->getRect().x || 
		rect.y!=owner->getRect().y ||
		actual_tick>max_tick)
	{
		if(actual_tick>max_tick)
		{
			actual_tick=0;
			actual_frame++;
			if(actual_frame>=num_frames)
				actual_frame=0;
		} 

		//Poner el fondo donde estaba antes
		g->rendermanager->addToDraw(rect);

		rect=owner->getRect();
		rect.w=get_w();
		rect.h=get_h();

		//Poner el fondo donde va a estar ahora
		g->rendermanager->addToDraw(rect);
	}
	actual_tick++;
}

void Sprite::desactivate()
{
	g->rendermanager->addToDraw(rect);
}

void Sprite::draw()
{
	SDL_Rect src={get_w()*actual_frame,0,get_w(),get_h()};
	SDL_Rect dst=rect;
	SDL_BlitSurface(img,&src,SDL_GetVideoSurface(),&dst);
}

void Sprite::draw(SDL_Rect &r)
{
	SDL_Rect src;
	SDL_Rect dst;

	dst=getRectCollision(rect,r);
	src.w=dst.w;
	src.h=dst.h;

	//Rellenamos los rectangulos src y dst
	//para pintar solo lo que se ha modificado
	//Si el actor esta a la derecha del rect
	if(r.x<=rect.x)
	{
		src.x=0;
	}
	//Si el actor esta a la izquierda del rect
	else
	{
		//Si el actor esta fuera de la pantalla
		src.x=r.x-rect.x;
	}

	//Si el actor esta por debajo del rectangulo
	if(r.y<=rect.y)
	{
		src.y=0;
	}
	//Si el actor esta por encima del rectangulo
	else
	{
		src.y=r.y-rect.y;
	}

	src.x=src.x+get_w()*actual_frame;

	SDL_BlitSurface(img,&src,SDL_GetVideoSurface(),&dst);
}
