#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>

#include <stdio.h>

enum zero_t {zero};

class Vector
{
	public:
		Vector(){}
		Vector(const zero_t):x(0),y(0){}
		Vector(const float ax,const float ay):x(ax),y(ay){}
		Vector(const Vector &v):x(v.x),y(v.y){}

		float scale_x()const{return x;}
		float scale_y()const{return y;}

		void set(const float ax,const float ay){x=ax;y=ay;}

		Vector operator+(const Vector &v)const{return Vector(x+v.x,y+v.y);}
		Vector operator-(const Vector &v)const{return Vector(x-v.x,y-v.y);}
		bool operator==(const Vector &v)const{return (x==v.x)&&(y==v.y);}
		bool operator!=(const Vector &v)const{return (x!=v.x)||(y!=v.y);}

		float module()const{return sqrt(x*x+y*y);}
		float angle()const{return atan2(y,x);}

		void imprime()const{printf("x: %f\ny: %f\n",x,y);}
	private:
		float x,y;
};

inline Vector add(const Vector &a,const Vector &b)
{
	return Vector(a.scale_x()+b.scale_x(),a.scale_y()+b.scale_y());
}

inline Vector sub(const Vector &a,const Vector &b)
{
	return Vector(a.scale_x()-b.scale_x(),a.scale_y()-b.scale_y());
}

inline float escalate(const Vector &a,const Vector &b)
{
	return a.scale_x()*b.scale_x()+a.scale_y()*b.scale_y();
}

inline Vector multiply(const Vector &v,const float &k)
{
	return Vector(v.scale_x()*k,v.scale_y()*k);
}

inline Vector multiply(const float &k,const Vector &v)
{
	return multiply(v,k);
}

inline Vector normalize(const Vector &v)
{
	return Vector(v.scale_x()/v.module(),v.scale_y()/v.module());
}

inline Vector reflect(const Vector &v,const Vector &n)
{
	Vector aux=normalize(n);
	float proyection=escalate(v,aux);
	aux=multiply(proyection,aux);
	Vector resta=sub(aux,v);
	return add(v,multiply(2,resta));
}

inline Vector reverse(const Vector &v)
{
	return multiply(-1,v);
}

inline Vector rotateRight(const Vector &v)
{
	return Vector(-v.scale_y(),v.scale_x());
}

inline Vector rotateLeft(const Vector &v)
{
	return Vector(v.scale_y(),-v.scale_x());
}
/*
inline Vector add(const Vector,const Vector)const;
inline Vector sub(const Vector,const Vector)const;
inline float escalate(const Vector,const Vector)const;
inline Vector multiply(const Vector,const float)const;
inline Vector multiply(const float,const Vector)const;
inline Vector normalize(const Vector)const;
Vector reflect(const Vector,const Vector)const;
*/

#endif
